// +-------------------------------------------------------------------+
// 		KCOMM - Serial modem control routines
// 		  (c)1993, 7 june by Kevin Phillips
// 	Uses Interrupt driven read routine and circular buffer
// +-------------------------------------------------------------------+


#include "globals.h"			// Include all globals,macros,etc.

// +-------------------------------------------------------------------+
//  Initialise Storage for old int vector
// +-------------------------------------------------------------------+

static void interrupt (far *oldvec) (void);

// +-------------------------------------------------------------------+
//  Serial port read and include to buffer routine
// +-------------------------------------------------------------------+

void interrupt myserial (void)
{
	disable();

	buffer[inptr++] =  inportb (moddata);	// Read data into buffer

	if (inptr == BUFFSIZ)	// Check for end of buffer
		inptr = 0;		// and loop back.

	enable();

	outportb (0x20,0x20);		// Signal EOI to 8259
}


// +-------------------------------------------------------------------+
//  Initialise Data and Interrupts. Called at start of program
// +-------------------------------------------------------------------+

void ser_init ()
{
	union REGS r;

	configuration = speedb + parityb + stopb + datab;

	r.x.dx = myport;		// First, Initialise port
	r.h.al = configuration;		// using BIOS routine
	r.h.ah = 0;
	int86 (0x14, &r, &r);

}

void set_interrupt()
{

	oldvec = getvect( intv );	// Store old vector of IRQ

	disable ();

	setvect (intv, myserial);	// Set up new vector to routine

	inportb (moddata);		// Clear port by reading it
	outportb (modcomd, 0x0b);	// Initialise DTR,RTS and enable interrupt
	outportb (modint, 0x01);	// Set DATA AVAILABLE interrupt


	outportb (INTCONT, (inportb (INTCONT) & ena8259));	// Enable IRQ

	enable ();
}


// +-------------------------------------------------------------------+
//  Serial shut-down routine to restore interrupt status
//  IMPORTANT - MUST BE CALLED WHEN YOU EXIT APPLICATION! (or CRASH!!)
// +-------------------------------------------------------------------+

void ser_close ()
{
	disable();			// Disable the interrupts

	outportb (INTCONT,(inportb(INTCONT) | dis8259)); // Disable Interrupt

	setvect (intv,oldvec);		// Restore old vector

	outportb (modcomd,0);		// Disable interrupt,DTR,RTS
	outportb (modint,0);		// Reset interrupts.

	enable();

}


// +-------------------------------------------------------------------+
//  Send a character out the serial port
// +-------------------------------------------------------------------+

void send_char (unsigned char c)
{
	while (!(inportb (linestat) & 0x20));	// Wait for all clear to transmit

	outportb (moddata,c);		// Transmit character
}


// +-------------------------------------------------------------------+
//  See if a character exists in the circular buffer, returning -1 if
//  none, else returning the character.
// +-------------------------------------------------------------------+

int rec_char ()
{
	int c;				// Temp storage

	if (inptr == outptr)
			return(-1);

		disable();

		c = buffer[outptr++];			// Read character

		if (outptr == BUFFSIZ)		// Check for end of buffer
			outptr = 0;   	// and loop back

		enable();

		return (c);			// Return the character code
}

// +-------------------------------------------------------------------+
//  Break by dropping DTR for 'dtime' m/secs
// +-------------------------------------------------------------------+

void hangupmodem(int dtime)
{
	outportb (modcomd, (inportb(modcomd) & 0xfe));	// Drop DTR
	delay (dtime);
	outportb (modcomd, (inportb(modcomd) | 1));	// Re-init DTR
}

// +-------------------------------------------------------------------+
//  Keyboard read routine
//  Returns : (0)-No keypress   (CODE)-keypress   (-CODE)-special key
// +-------------------------------------------------------------------+

int key_press()
{
	union inkey{ 			// Set up storage for keypress
		char ch[2];
		int i;
		} c;

	if (bioskey(1)) {		// If keypressed
		c.i = bioskey(0);	// Read the key

		if (c.ch[0])			// If normal, return ASCII
			return (c.ch[0]);
		if (c.ch[1])			// If special, retunr -CODE
			return (-c.ch[1]);
		}
	return (0);			// Exit for NO keypress.
}

//
// Set up configuration for comms port
//

void set_com()
{
		if (myport==0) {			// Initialise Variables COM1 (0)
		moddata = MODDAT1;
		modcomd = MODCOM1;
		modint  = MODINT1;
		linestat= LNESTA1;
		modstat = MODMST1;
		modiir  = MODIIR1;
		ena8259 = ENABLE1;
		dis8259 = DISABL1;
		intv    = INTVEC1;
		}
	else if (myport==1)			// or, COM2 (1)
		{
		moddata = MODDAT2;
		modcomd = MODCOM2;
		modint  = MODINT2;
		linestat= LNESTA2;
		modstat = MODMST2;
		modiir  = MODIIR2;
		ena8259 = ENABLE2;
		dis8259 = DISABL2;
		intv    = INTVEC2;
		}
       else if (myport==2) {			// or, COM3 (2)
		moddata = MODDAT3;
		modcomd = MODCOM3;
		modint  = MODINT3;
		linestat= LNESTA3;
		modstat = MODMST3;
		modiir  = MODIIR3;
		ena8259 = ENABLE1;
		dis8259 = DISABL1;
		intv    = INTVEC1;
		}
	else if (myport==3)			// or, COM4 (3)
		{
		moddata = MODDAT4;
		modcomd = MODCOM4;
		modint  = MODINT4;
		linestat= LNESTA4;
		modstat = MODMST4;
		modiir  = MODIIR4;
		ena8259 = ENABLE2;
		dis8259 = DISABL2;
		intv    = INTVEC2;
		}
}

// Send a String using * as return inside

void send_st(char *string)
{
	int i,j,k,l,m;

	i=strlen(string);		// Set up for looping through string
	j=0;

	while (j<i){
		k = *string++;		// Read Character

		if (k=='*'){       	// Is it '*' - RETURN ?
			send_char('\r');	// Yep, Send return
			delay (1000); 		// Give modem time to process
			}			// Else Characters get lost!
		else if (k != '\0')
				send_char(k);		// Else, Send the char
			j++;
		}
}

//
// BTERM main goes here !!!
//

void main()
{
	int keyb, dtb;
	char *textdta;
	char thend[80];

	vidmem = (char far *) 0xb8000000;	// Pointer to screen memory
	init_bterm();	// Initialise all details

	atxy(0,4);	// Set up cursor

	cls (SCRCOL);
	cstring (0,SCRCOL,"BTERM Communications application (c)1993 T42 Software, Kevin Phillips");
	cstring (2,SCRCOL,"ALT-Z for commands");

	send_st (modemcontrolstring);	// Signal Modem for setup


	while (btermquit){		// Main loop until btermquit = FALSE
		keyb = key_press();	// Read keyboard
		if (keyb > 0){		// If its alright (ASCII)
			send_char(keyb);	// Then send character
			if (lecho)		// And check for echo
				printf("%c",keyb);

			if (lnfeed && keyb=='\r'){	// Check for Linefeed
					send_char('\n');
					if (lecho)
						printf("\n");
					}
			} // End of 'if (key>0)'

		if (keyb < 0){			// Check for special keys
			keyb = abs (keyb);
			which_menu(keyb);	// Select menu
			}

		dtb = rec_char();		// Read incoming data from modem

		if (dtb != -1){			// Is anything there ?
			putchar(dtb);		// Yes, so print it!

			if (printerf)		// Check for printer on
				prnchar(dtb);	// and send to printer at LPT1
			}

		}	// End of main loop

// OK, Now the program has exited so we :

	hangupmodem(1000);	// Disconnect Modem
	ser_close();	// Switch off the Interrupt
	free (buffer);	// Free the buffer memory

	cls (SCRCOL);	// Clear the screen
	atxy (0,21);	// Cursor = bottom of screen

	// And print the registration message at the end

	cstring(0,BLUWHT,"BTERM V1.0   Copyright (c) 1993 -  T42 Software");
	cstring(1,BLUYEL,"<<   PO Box 7116, Wellesley St., Auckland, New Zealand   >>");
	cstring(3,SCRCOL,"BTERM is a FREEWARE product. That is, it requires NO fee!");
	cstring(4,SCRCOL,"We hope you will enjoy using BTERM.  Future versions will");
	cstring(5,SCRCOL,"contain functions not present in this version.");
	cstring(6,SCRCOL,"(UP/DOWNLOAD facilities, Terminal emulation, Host modes,etc)");
	cstring(8,BLKYEL,"If you have any ideas for functions that you would like to");
	cstring(9,BLKYEL,"see included in future versions of BTERM,PLEASE LET US KNOW !");
	cstring(10,BLKYEL,"(Same goes for New products. What do YOU want?)");
	cstring(11,BLK BAK + RED + blinking,"ALL SUGGESTIONS TO THE ABOVE ADDRESS PLEASE.");
	cstring(13,BLKWHT,"A disk containing copies of other T42 Software products");
	cstring(14,BLKWHT,"(Unregistered Shareware & Freeware) can be purchased for :");
	sprintf(thend,"US $8.00, NZ $12.00, AS $10.00, UK %c5.00",156);
	cstring(15,BLKCYN,thend);
	cstring(16,BLUYEL,"(BANK CHEQUE or MONEY ORDER made out to Kevin Phillips)");
	cstring(17,BLKCYN,"To the above address.");
	cstring(19,BLUWHT,"PLEASE HELP THE DISTRIBUTION OF THIS SOFTWARE BY COPYING IT");
	cstring(20,BLUYEL,"TO FRIENDS, BBS's, Etc. - SUPPORT THE SHAREWARE & FREEWARE CONCEPT");


	SNDFLG = TRUE;

	alarm();	// Signal End, and exit
}
